package ${basePackage}.module.system.mgr;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ${basePackage}.frame.base.ErrorType;
import ${basePackage}.frame.base.Token;
import ${basePackage}.frame.utils.IDgenerator;
import ${basePackage}.frame.utils.MapperUtil;
import ${basePackage}.frame.utils.Message;
import ${basePackage}.frame.utils.ValidationUtil;
import ${basePackage}.module.system.ent.RoleResource;
import ${basePackage}.module.system.mpr.RoleResourceMapper;
import ${basePackage}.module.system.req.RoleResourceCreateRequest;
import ${basePackage}.module.system.req.RoleResourceDeleteRequest;
import ${basePackage}.module.system.req.RoleResourceFindRequest;
import ${basePackage}.module.system.req.RoleResourceGetRequest;
import ${basePackage}.module.system.req.RoleResourceUpdateRequest;
import ${basePackage}.module.system.rsp.RoleResourceCreateResponse;
import ${basePackage}.module.system.rsp.RoleResourceDeleteResponse;
import ${basePackage}.module.system.rsp.RoleResourceFindResponse;
import ${basePackage}.module.system.rsp.RoleResourceGetResponse;
import ${basePackage}.module.system.rsp.RoleResourceUpdateResponse;

/**
 * ROLE_RESOURCE - 角色资源关系
 *
 * @author wangbing
 * @version 0.0.1
 * @since 2017-01-01
 */
@Transactional
@Service
public class RoleResourceManagerImpl implements RoleResourceManager {

    @Autowired
    private RoleResourceMapper roleResourceMapper;

    /**
     * 插入
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public RoleResourceCreateResponse create(RoleResourceCreateRequest request, Token token) {
        RoleResourceCreateResponse response = new RoleResourceCreateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long id = IDgenerator.nextId();
        RoleResource entity = MapperUtil.map(request, RoleResource.class);
        entity.setId(id);

        long result = roleResourceMapper.insert(entity, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.CREATE_FAILURE);
            return response;
        }
        response.setId(id);

        return response;
    }

    /**
     * 逻辑删除
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public RoleResourceDeleteResponse delete(RoleResourceDeleteRequest request, Token token) {
        RoleResourceDeleteResponse response = new RoleResourceDeleteResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = roleResourceMapper.delete(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.DELETE_FAILURE);
            return response;
        }
        response.setResult(result);

        return response;
    }

    /**
     * 更新
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public RoleResourceUpdateResponse update(RoleResourceUpdateRequest request, Token token) {
        RoleResourceUpdateResponse response = new RoleResourceUpdateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = roleResourceMapper.update(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.UPDATE_FAILURE);
            return response;
        }
        response.setResult(result);

        return response;
    }

    /**
     * 查询
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public RoleResourceFindResponse find(RoleResourceFindRequest request, Token token) {
        RoleResourceFindResponse response = new RoleResourceFindResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        PageHelper.startPage(request.getPageNumber(), request.getPageSize());
        if (StringUtil.isNotEmpty(request.getSortKey())) {
            PageHelper.orderBy(request.getSortKey() + " " + request.getSortType());
        }
        PageInfo<RoleResource> pageInfo = new PageInfo<>(roleResourceMapper.find(request, token));

        response.setResult(pageInfo.getList());
        response.setTotalCount(pageInfo.getTotal());

        return response;
    }

    /**
     * 获得对象
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public RoleResourceGetResponse get(RoleResourceGetRequest request, Token token) {
        RoleResourceGetResponse response = new RoleResourceGetResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        RoleResource po = roleResourceMapper.get(request, token);

        if (po != null) {
            response.setRoleResource(po);
        } else {
            response.addError(ErrorType.BUSINESS_ERROR, Message.GET_FAILURE);
        }

        return response;
    }
}
