package ${basePackage}.module.system.mgr;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ${basePackage}.config.CacheConfig;
import ${basePackage}.frame.base.ErrorType;
import ${basePackage}.frame.base.Token;
import ${basePackage}.frame.utils.IDgenerator;
import ${basePackage}.frame.utils.MapperUtil;
import ${basePackage}.frame.utils.Message;
import ${basePackage}.frame.utils.ValidationUtil;
import ${basePackage}.module.system.ent.Tokens;
import ${basePackage}.module.system.mpr.TokensMapper;
import ${basePackage}.module.system.req.TokensBuildRequest;
import ${basePackage}.module.system.req.TokensCreateRequest;
import ${basePackage}.module.system.req.TokensDeleteRequest;
import ${basePackage}.module.system.req.TokensFindRequest;
import ${basePackage}.module.system.req.TokensGetRequest;
import ${basePackage}.module.system.req.TokensUpdateRequest;
import ${basePackage}.module.system.rsp.TokensBuildResponse;
import ${basePackage}.module.system.rsp.TokensCreateResponse;
import ${basePackage}.module.system.rsp.TokensDeleteResponse;
import ${basePackage}.module.system.rsp.TokensFindResponse;
import ${basePackage}.module.system.rsp.TokensGetResponse;
import ${basePackage}.module.system.rsp.TokensUpdateResponse;

/**
 * TOKENS - 通行证
 *
 * @author author
 * @version 0.0.1
 * @since 2017-01-01
 */
@Transactional
@Service
public class TokensManagerImpl implements TokensManager {

    @Value("${r'${web.url.auth.admin}'}")
    private String admin;
    @Autowired
    private TokensMapper tokensMapper;

    /**
     * 插入
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public TokensCreateResponse create(TokensCreateRequest request, Token token) {
        TokensCreateResponse response = new TokensCreateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long id = IDgenerator.nextId();
        Tokens entity = MapperUtil.map(request, Tokens.class);
        entity.setId(id);

        long result = tokensMapper.insert(entity, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.CREATE_FAILURE);
            return response;
        }
        response.setId(id);

        return response;
    }

    /**
     * 逻辑删除
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public TokensDeleteResponse delete(TokensDeleteRequest request, Token token) {
        TokensDeleteResponse response = new TokensDeleteResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = tokensMapper.delete(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.DELETE_FAILURE);
            return response;
        }
        response.setResult(result);

        return response;
    }

    /**
     * 更新
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public TokensUpdateResponse update(TokensUpdateRequest request, Token token) {
        TokensUpdateResponse response = new TokensUpdateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = tokensMapper.update(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.UPDATE_FAILURE);
            return response;
        }
        response.setResult(result);

        return response;
    }

    /**
     * 查询
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public TokensFindResponse find(TokensFindRequest request, Token token) {
        TokensFindResponse response = new TokensFindResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        PageHelper.startPage(request.getPageNumber(), request.getPageSize());
        if (StringUtil.isNotEmpty(request.getSortKey())) {
            PageHelper.orderBy(request.getSortKey() + " " + request.getSortType());
        }
        PageInfo<Tokens> pageInfo = new PageInfo<>(tokensMapper.find(request, token));

        response.setResult(pageInfo.getList());
        response.setTotalCount(pageInfo.getTotal());

        return response;
    }

    /**
     * 获得对象
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public TokensGetResponse get(TokensGetRequest request, Token token) {
        TokensGetResponse response = new TokensGetResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        Tokens po = tokensMapper.get(request, token);

        if (po != null) {
            response.setTokens(po);
        } else {
            response.addError(ErrorType.BUSINESS_ERROR, Message.GET_FAILURE);
        }

        return response;
    }

    /**
     * 构建Token
     *
     * @param request 请求对象
     * @param token   令牌
     * @return
     */
    @Override
    @Cacheable(value = CacheConfig.TOKEN_CACHE, key = "#request.token")
    public TokensBuildResponse build(TokensBuildRequest request, Token token) {
        TokensBuildResponse response = new TokensBuildResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        TokensFindRequest tokensFindRequest = MapperUtil.map(request, TokensFindRequest.class);
        tokensFindRequest.setValid(true);
        TokensFindResponse tokensFindResponse = this.find(tokensFindRequest, token);
        if (tokensFindResponse.hasError()) {
            response.addErrors(tokensFindResponse.getErrors());
            return response;
        }

        if (tokensFindResponse.getResult().size() == 0) {
            response.addError(ErrorType.BUSINESS_ERROR, "令牌无效或不存在");
            return response;
        } else if (tokensFindResponse.getResult().size() > 1) {
            response.addError(ErrorType.BUSINESS_ERROR, "令牌异常");
            return response;
        } else {
            Tokens tokens = tokensFindResponse.getResult().get(0);
            Token newToken = new Token();
            if (tokens.getUserName().equals(admin)) {
                newToken.setId(0);
                newToken.setUserId(0);
                newToken.setUserName(admin);
                newToken.putResource(".*");
            } else {
                newToken.setId(tokens.getId());
                newToken.setUserId(tokens.getUserId());
                newToken.setUserName(tokens.getUserName());
            }

            response.setToken(newToken);
        }

        return response;
    }
}
