package ${basePackage}.system;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import static org.junit.Assert.assertTrue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import ${basePackage}.module.system.mgr.ResourceManager;
import ${basePackage}.module.system.req.*;
import ${basePackage}.module.system.rsp.*;
import ${basePackage}.frame.base.Token;


/**
 * ResourceTest -  - 资源测试用例
 *
 * @author author
 * @version 0.0.1
 * @since 2017-01-01
 */
@RunWith(SpringRunner.class)
@SpringBootTest
@Transactional
public class ResourceTest {

    @Autowired
    private Token token;

    @Autowired
    private ResourceManager resourceManager;

    @Test
    public void testCreate() {
        ResourceCreateRequest request = new ResourceCreateRequest();
        request.setValue("资源内容");
        request.setComment("资源描述");
        request.setResourceType("code");
        request.setValid(true);

        ResourceCreateResponse response = resourceManager.create(request,token);

        assertTrue(!response.hasError());
    }

    @Test
    public void testDelete() {

        //创建数据
        ResourceCreateRequest createRequest = new ResourceCreateRequest();
        createRequest.setValue("资源内容");
        createRequest.setComment("资源描述");
        createRequest.setResourceType("code");
        createRequest.setValid(true);

        ResourceCreateResponse createResponse = resourceManager.create(createRequest,token);

        assertTrue(!createResponse.hasError() && createResponse.getId() > 0);
        //删除数据
        ResourceDeleteRequest request = new ResourceDeleteRequest();
        request.setId(createResponse.getId());

        ResourceDeleteResponse response = resourceManager.delete(request,token);

        assertTrue(!response.hasError() && response.getResult() == 1L);
    }

    @Test
    public void testUpdate() {
        //创建数据
        ResourceCreateRequest createRequest = new ResourceCreateRequest();
        createRequest.setValue("资源内容");
        createRequest.setComment("资源描述");
        createRequest.setResourceType("code");
        createRequest.setValid(true);

        ResourceCreateResponse createResponse = resourceManager.create(createRequest, token);

        assertTrue(!createResponse.hasError());

        //更新数据
        ResourceUpdateRequest request = new ResourceUpdateRequest();
        request.setId(createResponse.getId());
        request.setValue("资源内容");
        request.setComment("资源描述");
        request.setResourceType("code");
        request.setValid(true);

        ResourceUpdateResponse response = resourceManager.update(request,token);

        assertTrue(!response.hasError() && response.getResult() == 1L);
    }

    @Test
    public void testFind() {
        ResourceFindRequest request = new ResourceFindRequest();
        request.setValue("资源内容");
        request.setComment("资源描述");
        request.setResourceType("code");
        request.setValid(true);

        ResourceFindResponse response = resourceManager.find(request,token);

        assertTrue(!response.hasError());
    }

    @Test
    public void testGet() {
        //创建数据
        ResourceCreateRequest createRequest = new ResourceCreateRequest();
        createRequest.setValue("资源内容");
        createRequest.setComment("资源描述");
        createRequest.setResourceType("code");
        createRequest.setValid(true);

        ResourceCreateResponse createResponse = resourceManager.create(createRequest, token);

        assertTrue(!createResponse.hasError());

        //获得数据
        ResourceGetRequest request = new ResourceGetRequest();
        request.setId(createResponse.getId());

        ResourceGetResponse response = resourceManager.get(request,token);

        assertTrue(!response.hasError() && response.getResource() != null);
    }
}
